/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
module org.libsdl.video;

import org.libsdl.stdinc;
import org.libsdl.rwops;

immutable ubyte SDL_ALPHA_OPAQUE      = 255;
immutable ubyte SDL_ALPHA_TRANSPARENT = 0;

struct SDL_Rect
{
	Sint16 x, y;
	Uint16 w, h;
}

struct SDL_Color
{
	Uint8 r;
	Uint8 g;
	Uint8 b;
	Uint8 unused;
}
alias SDL_Color SDL_Colour;

struct SDL_Palette
{
	int ncolors;
	SDL_Color* colors;
}

struct SDL_PixelFormat
{
	SDL_Palette* palette;
	Uint8  BitsPerPixel;
	Uint8  BytesPerPixel;
	Uint8  Rloss;
	Uint8  Gloss;
	Uint8  Bloss;
	Uint8  Aloss;
	Uint8  Rshift;
	Uint8  Gshift;
	Uint8  Bshift;
	Uint8  Ashift;
	Uint32 Rmask;
	Uint32 Gmask;
	Uint32 Bmask;
	Uint32 Amask;
	Uint32 colorkey;
	Uint8  alpha;
}

struct private_hwdata;
struct SDL_BlitMap;

struct SDL_Surface
{
	Uint32 flags;
	SDL_PixelFormat* format;
	int w, h;
	Uint16 pitch;
	void* pixels;
	int offset;
	private_hwdata* hwdata;
	SDL_Rect clip_rect;
	Uint32 unused1;
	Uint32 locked;
	SDL_BlitMap* map;
	uint format_version;
	int refcount;
}

immutable Uint32 SDL_SWSURFACE = 0x00000000;
immutable Uint32 SDL_HWSURFACE = 0x00000001;
immutable Uint32 SDL_ASYNCBLIT = 0x00000004;

immutable Uint32 SDL_ANYFORMAT	= 0x10000000;
immutable Uint32 SDL_HWPALETTE	= 0x20000000;
immutable Uint32 SDL_DOUBLEBUF	= 0x40000000;
immutable Uint32 SDL_FULLSCREEN	= 0x80000000;
immutable Uint32 SDL_OPENGL     = 0x00000002;
immutable Uint32 SDL_OPENGLBLIT	= 0x0000000A;
immutable Uint32 SDL_RESIZABLE	= 0x00000010;
immutable Uint32 SDL_NOFRAME	= 0x00000020;

immutable Uint32 SDL_HWACCEL     = 0x00000100;
immutable Uint32 SDL_SRCCOLORKEY = 0x00001000;
immutable Uint32 SDL_RLEACCELOK	 = 0x00002000;
immutable Uint32 SDL_RLEACCEL    = 0x00004000;
immutable Uint32 SDL_SRCALPHA    = 0x00010000;
immutable Uint32 SDL_PREALLOC    = 0x01000000;

bool SDL_MUSTLOCK(SDL_Surface* surface)
{
	return surface.offset || (surface.flags & (SDL_HWSURFACE | SDL_ASYNCBLIT | SDL_RLEACCEL)) != 0;
}

extern (C)
{
	alias int function(SDL_Surface* src, SDL_Rect* srcrect, SDL_Surface* dst, SDL_Rect* dstrect) SDL_blit;
}

struct SDL_VideoInfo
{
	Uint32 bitfield;
	Uint32 video_mem;
	SDL_PixelFormat* vfmt;
	int current_w;
	int current_h;

	@property Uint32 hw_available() { return (bitfield >> 31) & 0b1; }
	@property Uint32 wm_available() { return (bitfield >> 30) & 0b1; }
	@property Uint32 UnusedBits1()  { return (bitfield >> 24) & 0b111111; }
	@property Uint32 UnusedBits2()  { return (bitfield >> 23) & 0b1; }
	@property Uint32 blit_hw()      { return (bitfield >> 22) & 0b1; }
	@property Uint32 blit_hw_CC()   { return (bitfield >> 21) & 0b1; }
	@property Uint32 blit_hw_A()    { return (bitfield >> 20) & 0b1; }
	@property Uint32 blit_sw()      { return (bitfield >> 19) & 0b1; }
	@property Uint32 blit_sw_CC()   { return (bitfield >> 18) & 0b1; }
	@property Uint32 blit_sw_A()    { return (bitfield >> 17) & 0b1; }
	@property Uint32 blit_fill()    { return (bitfield >> 16) & 0b1; }
	@property Uint32 UnusedBits3()  { return bitfield & 0b1111111111111111; }

	@property void hw_available(Uint32 b) { bitfield = (bitfield & ~0b10000000000000000000000000000000) | ((b & 0b1) << 31); }
	@property void wm_available(Uint32 b) { bitfield = (bitfield & ~0b01000000000000000000000000000000) | ((b & 0b1) << 30); }
	@property void UnusedBits1(Uint32 b)  { bitfield = (bitfield & ~0b00111111000000000000000000000000) | ((b & 0b111111) << 24); }
	@property void UnusedBits2(Uint32 b)  { bitfield = (bitfield & ~0b00000000100000000000000000000000) | ((b & 0b1) << 23); }
	@property void blit_hw(Uint32 b)      { bitfield = (bitfield & ~0b00000000010000000000000000000000) | ((b & 0b1) << 22); }
	@property void blit_hw_CC(Uint32 b)   { bitfield = (bitfield & ~0b00000000001000000000000000000000) | ((b & 0b1) << 21); }
	@property void blit_hw_A(Uint32 b)    { bitfield = (bitfield & ~0b00000000000100000000000000000000) | ((b & 0b1) << 20); }
	@property void blit_sw(Uint32 b)      { bitfield = (bitfield & ~0b00000000000010000000000000000000) | ((b & 0b1) << 19); }
	@property void blit_sw_CC(Uint32 b)   { bitfield = (bitfield & ~0b00000000000001000000000000000000) | ((b & 0b1) << 18); }
	@property void blit_sw_A(Uint32 b)    { bitfield = (bitfield & ~0b00000000000000100000000000000000) | ((b & 0b1) << 17); }
	@property void blit_fill(Uint32 b)    { bitfield = (bitfield & ~0b00000000000000010000000000000000) | ((b & 0b1) << 16); }
	@property void UnusedBits3(Uint32 b)  { bitfield = (bitfield & ~0b1111111111111111) | (b & 0b1111111111111111); }
}

immutable Uint32 SDL_YV12_OVERLAY = 0x32315659;
immutable Uint32 SDL_IYUV_OVERLAY = 0x56555949;
immutable Uint32 SDL_YUY2_OVERLAY = 0x32595559;
immutable Uint32 SDL_UYVY_OVERLAY = 0x59565955;
immutable Uint32 SDL_YVYU_OVERLAY = 0x55595659;

struct private_yuvhwfuncs;
struct private_yuvhwdata;

struct SDL_Overlay
{
	Uint32 format;
	int w, h;
	int planes;
	Uint16* pitches;
	Uint8** pixels;
	private_yuvhwfuncs* hwfuncs;
	private_yuvhwdata* hwdata;
	Uint32 bitfield;

	@property Uint32 hw_overlay() { return bitfield >> 31; }
	@property Uint32 UnusedBits() { return bitfield & 0x7fffffff; }

	@property void hw_overlay(Uint32 b) { bitfield = (bitfield & 0x7fffffff) | (b << 31); }
	@property void UnusedBits(Uint32 b) { bitfield = (bitfield & 0x80000000) | (b & 0x7fffffff); }
}

enum SDL_GLattr
{
    SDL_GL_RED_SIZE,
    SDL_GL_GREEN_SIZE,
    SDL_GL_BLUE_SIZE,
    SDL_GL_ALPHA_SIZE,
    SDL_GL_BUFFER_SIZE,
    SDL_GL_DOUBLEBUFFER,
    SDL_GL_DEPTH_SIZE,
    SDL_GL_STENCIL_SIZE,
    SDL_GL_ACCUM_RED_SIZE,
    SDL_GL_ACCUM_GREEN_SIZE,
    SDL_GL_ACCUM_BLUE_SIZE,
    SDL_GL_ACCUM_ALPHA_SIZE,
    SDL_GL_STEREO,
    SDL_GL_MULTISAMPLEBUFFERS,
    SDL_GL_MULTISAMPLESAMPLES,
    SDL_GL_ACCELERATED_VISUAL,
    SDL_GL_SWAP_CONTROL
}

alias SDL_GLattr.SDL_GL_RED_SIZE SDL_GL_RED_SIZE;
alias SDL_GLattr.SDL_GL_GREEN_SIZE SDL_GL_GREEN_SIZE;
alias SDL_GLattr.SDL_GL_BLUE_SIZE SDL_GL_BLUE_SIZE;
alias SDL_GLattr.SDL_GL_ALPHA_SIZE SDL_GL_ALPHA_SIZE;
alias SDL_GLattr.SDL_GL_BUFFER_SIZE SDL_GL_BUFFER_SIZE;
alias SDL_GLattr.SDL_GL_DOUBLEBUFFER SDL_GL_DOUBLEBUFFER;
alias SDL_GLattr.SDL_GL_DEPTH_SIZE SDL_GL_DEPTH_SIZE;
alias SDL_GLattr.SDL_GL_STENCIL_SIZE SDL_GL_STENCIL_SIZE;
alias SDL_GLattr.SDL_GL_ACCUM_RED_SIZE SDL_GL_ACCUM_RED_SIZE;
alias SDL_GLattr.SDL_GL_ACCUM_GREEN_SIZE SDL_GL_ACCUM_GREEN_SIZE;
alias SDL_GLattr.SDL_GL_ACCUM_BLUE_SIZE SDL_GL_ACCUM_BLUE_SIZE;
alias SDL_GLattr.SDL_GL_ACCUM_ALPHA_SIZE SDL_GL_ACCUM_ALPHA_SIZE;
alias SDL_GLattr.SDL_GL_STEREO SDL_GL_STEREO;
alias SDL_GLattr.SDL_GL_MULTISAMPLEBUFFERS SDL_GL_MULTISAMPLEBUFFERS;
alias SDL_GLattr.SDL_GL_MULTISAMPLESAMPLES SDL_GL_MULTISAMPLESAMPLES;
alias SDL_GLattr.SDL_GL_ACCELERATED_VISUAL SDL_GL_ACCELERATED_VISUAL;
alias SDL_GLattr.SDL_GL_SWAP_CONTROL SDL_GL_SWAP_CONTROL;

immutable Uint8 SDL_LOGPAL  = 0x01;
immutable Uint8 SDL_PHYSPAL = 0x02;

extern (C)
{
	int SDL_VideoInit(const(char)* driver_name, Uint32 flags);
	void SDL_VideoQuit();
	char* SDL_VideoDriverName(char* namebuf, int maxlen);
	SDL_Surface* SDL_GetVideoSurface();
	const(SDL_VideoInfo)* SDL_GetVideoInfo();
	int SDL_VideoModeOK(int width, int height, int bpp, Uint32 flags);
	SDL_Rect** SDL_ListModes(SDL_PixelFormat* format, Uint32 flags);
	SDL_Surface* SDL_SetVideoMode(int width, int height, int bpp, Uint32 flags);
	void SDL_UpdateRects(SDL_Surface* screen, int numrects, SDL_Rect* rects);
	void SDL_UpdateRect(SDL_Surface* screen, Sint32 x, Sint32 y, Uint32 w, Uint32 h);
	int SDL_Flip(SDL_Surface* screen);
	int SDL_SetGamma(float red, float green, float blue);
	int SDL_SetGammaRamp(const(Uint16)* red, const(Uint16)* green, const(Uint16)* blue);
	int SDL_GetGammaRamp(Uint16* red, Uint16* green, Uint16* blue);
	int SDL_SetColors(SDL_Surface* surface, SDL_Color* colors, int firstcolor, int ncolors);
	int SDL_SetPalette(SDL_Surface* surface, int flags, SDL_Color* colors, int firstcolor, int ncolors);
	Uint32 SDL_MapRGB(const(SDL_PixelFormat*) format, Uint8 r, Uint8 g, Uint8 b);
	Uint32 SDL_MapRGBA(const(SDL_PixelFormat*) format, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
	void SDL_GetRGB(Uint32 pixel, const(SDL_PixelFormat*) fmt, Uint8* r, Uint8* g, Uint8* b);
	void SDL_GetRGBA(Uint32 pixel, const(SDL_PixelFormat*) fmt, Uint8* r, Uint8* g, Uint8* b, Uint8* a);
	SDL_Surface* SDL_CreateRGBSurface(Uint32 flags, int width, int height, int depth, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
	alias SDL_CreateRGBSurface SDL_AllocSurface;
	SDL_Surface* SDL_CreateRGBSurfaceFrom(void* pixels, int width, int height, int depth, int pitch, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
	void SDL_FreeSurface(SDL_Surface* surface);
	int SDL_LockSurface(SDL_Surface* surface);
	void SDL_UnlockSurface(SDL_Surface* surface);
	SDL_Surface* SDL_LoadBMP_RW(SDL_RWops* src, int freesrc);
}

SDL_Surface* SDL_LoadBMP(const(char)* file)
{
	return SDL_LoadBMP_RW(SDL_RWFromFile(file, "rb"), 1);
}

extern (C)
{
	int SDL_SaveBMP_RW(SDL_Surface* surface, SDL_RWops* dst, int freedst);
}

int SDL_SaveBMP(SDL_Surface* surface, const(char)* file)
{
	return SDL_SaveBMP_RW(surface, SDL_RWFromFile(file, "wb"), 1);
}

extern (C)
{
	int SDL_SetColorKey(SDL_Surface* surface, Uint32 flag, Uint32 key);
	int SDL_SetAlpha(SDL_Surface* surface, Uint32 flag, Uint8 alpha);
	SDL_bool SDL_SetClipRect(SDL_Surface* surface, const(SDL_Rect)* rect);
	void SDL_GetClipRect(SDL_Surface* surface, SDL_Rect* rect);
	SDL_Surface* SDL_ConvertSurface(SDL_Surface* src, SDL_PixelFormat* fmt, Uint32 flags);
	int SDL_UpperBlit(SDL_Surface* src, SDL_Rect* srcrect, SDL_Surface* dst, SDL_Rect* dstrect);
	alias SDL_UpperBlit SDL_BlitSurface;
	int SDL_LowerBlit(SDL_Surface* src, SDL_Rect* srcrect, SDL_Surface* dst, SDL_Rect* dstrect);
	int SDL_FillRect(SDL_Surface* dst, SDL_Rect* dstrect, Uint32 color);
	SDL_Surface* SDL_DisplayFormat(SDL_Surface* surface);
	SDL_Surface* SDL_DisplayFormatAlpha(SDL_Surface* surface);
	SDL_Overlay* SDL_CreateYUVOverlay(int width, int height, Uint32 format, SDL_Surface *display);
	int SDL_LockYUVOverlay(SDL_Overlay* overlay);
	void SDL_UnlockYUVOverlay(SDL_Overlay* overlay);
	int SDL_DisplayYUVOverlay(SDL_Overlay* overlay, SDL_Rect* dstrect);
	void SDL_FreeYUVOverlay(SDL_Overlay* overlay);
	int SDL_GL_LoadLibrary(const(char)* path);
	void* SDL_GL_GetProcAddress(const(char)* proc);
	int SDL_GL_SetAttribute(SDL_GLattr attr, int value);
	int SDL_GL_GetAttribute(SDL_GLattr attr, int* value);
	void SDL_GL_SwapBuffers();
	void SDL_GL_UpdateRects(int numrects, SDL_Rect* rects);
	void SDL_GL_Lock();
	void SDL_GL_Unlock();
	void SDL_WM_SetCaption(const(char)* title, const(char)* icon);
	void SDL_WM_GetCaption(char** title, char** icon);
	void SDL_WM_SetIcon(SDL_Surface* icon, Uint8* mask);
	int SDL_WM_IconifyWindow();
	int SDL_WM_ToggleFullScreen(SDL_Surface* surface);
}

enum SDL_GrabMode
{
	SDL_GRAB_QUERY = -1,
	SDL_GRAB_OFF = 0,
	SDL_GRAB_ON = 1,
	SDL_GRAB_FULLSCREEN,
}

alias SDL_GrabMode.SDL_GRAB_QUERY SDL_GRAB_QUERY;
alias SDL_GrabMode.SDL_GRAB_OFF SDL_GRAB_OFF;
alias SDL_GrabMode.SDL_GRAB_ON SDL_GRAB_ON;
alias SDL_GrabMode.SDL_GRAB_FULLSCREEN SDL_GRAB_FULLSCREEN;

extern (C)
{
	SDL_GrabMode SDL_WM_GrabInput(SDL_GrabMode mode);
	int SDL_SoftStretch(SDL_Surface* src, SDL_Rect* srcrect, SDL_Surface* dst, SDL_Rect* dstrect);
}
